@@ -1,11 +1,10 @@
 import asyncio
-from collections import OrderedDict
 from io import BytesIO
 from typing import Literal
 
 from pyrogram import Client
 from pyrogram.errors import AuthKeyUnregistered, Unauthorized
-from pyrogram.filters import chat, contact, AndFilter
+from pyrogram.filters import chat, contact
 from pyrogram.handlers import MessageHandler
 from pyrogram.types import Message, InlineKeyboardMarkup, InlineKeyboardButton, ReplyKeyboardMarkup, KeyboardButton
 from x_auth.models import Session, App, Username
@@ -41,17 +40,18 @@ class BaseClient(Client):
             session = await Session.create(id=self.uid, api=app, user=username, is_bot=bt)
             self.api_id = app.id
             self.api_hash = app.hsh
-            await session.fetch_related("api")
+            # await session.fetch_related("api")
         self.storage.session = session
 
     async def start(self, use_qr: bool = False, except_ids: list[int] = None):
         if not self.is_connected:
             await self.preload()
             try:
-                await super().start(use_qr=use_qr, except_ids=except_ids or [])
+                return await super().start(use_qr=use_qr, except_ids=except_ids or [])
             except (AuthKeyUnregistered, Unauthorized) as e:
                 await self.storage.session.delete()
                 raise e
+        return self
 
     async def send(
         self,
@@ -77,33 +77,26 @@ class BaseClient(Client):
         else:
             return await self.send_message(uid, txt, reply_markup=ikm)
 
-    async def wait_from(self, uid: int, topic: str, past: int = 0, timeout: int = 10) -> str | None:
+    def subscribe_for(self, uid: int, topic: str) -> tuple[MessageHandler, int]:
         fltr = chat(uid)
         if topic == "phone":
             fltr &= contact
-        handler = MessageHandler(self.got_msg, fltr)
-        # handler, g = self.add_handler(handler, 1)
-        g = 0
-        if g not in self.dispatcher.groups:
-            self.dispatcher.groups[g] = []
-            self.dispatcher.groups = OrderedDict(sorted(self.dispatcher.groups.items()))
-        self.dispatcher.groups[g].append(handler)
         self.storage.session.state |= {uid: {"waiting_for": topic}}
-        #
+        return self.add_handler(MessageHandler(self.got_msg, fltr))
+
+    async def wait_from(
+        self, uid: int, topic: str, hg: tuple[MessageHandler, int], past: int = 0, timeout: int = 10
+    ) -> str | None:
         while past < timeout:
             if txt := self.storage.session.state.get(uid, {}).pop(topic, None):
-                # self.remove_handler(handler)
-                self.dispatcher.groups[g].remove(handler)
-                #
+                self.remove_handler(*hg)
                 return txt
-            await asyncio.sleep(1)
-            past += 1
-        return self.remove_handler(handler, g)
+            await asyncio.sleep(2)
+            past += 2
+        return self.remove_handler(*hg)
 
     async def got_msg(self, _, msg: Message):
         if tpc := self.storage.session.state.get(msg.from_user.id, {}).pop("waiting_for", None):
-            self.storage.session.state[msg.from_user.id][tpc] = msg.contact.phone_number if tpc == "phone" else msg.text
-
-    def rm_handler(self, uid: int):
-        for gi, grp in self.dispatcher.groups.items():
-            [self.remove_handler(h, gi) for h in grp if isinstance(h.filters, AndFilter) and uid in h.filters.base]
+            self.storage.session.state[msg.from_user.id] = {
+                tpc: msg.contact.phone_number if tpc == "phone" else msg.text
+            }
